



# liquidSVM Architecture

This document describes the architecture and design decisions behind liquidSVM.

The following ideas enable liquidSVM's performance:

- learning without offset: the offset of an SVM is on one hand not needed for infinite-dimensional kernels like the Gaussian RBF and on the other gives much more flexibility for transfer of solutions.
- cross-validation integrated in the core, because most of the calculations done for one hyperparameter-pair can be reused for the next! Also the solution is a very good warm-start for the next pair.
- getting benefits of modern CPUs by vectorizing and multi-threading calculations, also GPUs can be used.
- reusing the same building blocks for different learnings scenarios.
- scalability: spatial partitioning is 
- good default values so SVMs should work out of the box for most cases, however there are several levels of configurations for advanced users or even SVM-experts.

Most of these ideas are not only empirically verified but even mathematically proved.

## Overview

All calculations are iterated in different levels:

- task: e.g. for multi-class classification, every binary classification problem is a task, e.g. there are 6 one-vs-one tasks
for a problem with labels a,b,c,d. For the a-vs.-b task only samples for those two classes are selected.
- cell: if tasks are too big to use a global solution it can be split into smaller chunks - the best way is to use spatial cells.
- fold: every cell is split into k folds for the cross validation. The Gram matrix has only to be caldulated once per fold - this takes the longest time in the calculation.
- bandwidth parameter gamma: for every gamma, the Gram matrix is used to calculate the Kernel matrix.
- cost parameter lambda: the inner most iteration is the fastest one.

Observe that this means that every fold usually will have slightly different optimal hyper-parameters. Thus in inference there either has to be solved with the best hyper-parameters for the whole data set or a voting is used among the best hyper-parameters of every fold (default).

The general work of liquidSVM is split into three phases:

- train - the work horse: calculates solutions for all given tasks and hyper-parameter combinations.
- select - the cherry picker: this evaluates the best solutions based on the cross-validation results, and can retrain a final solution for the best hyper-parameters (if preferred).
- test - the inference step: if no retraining was done the solution of every fold is used for a final voting (default).

This architecture entails many decisions that can be configured if needed, e.g.:

- retrain: Should there be a retraining on all of the samples.
- store solutions internally: the train phase produces lots of solutions, of which only a hand-full will be considered in select and test. So if memory is low, all the solutions can be thrown away during the train phase and only the needed ones will be recalculated during the select.


## Configuration

There are several layers of configuration that correspond to levels of expert usage:

- Scenario selection: Classification, Regression, Quantiles, Expecties, ROC, Neyman-Pearson Learning
- High level configuration: Decisions regarding trade-offs Accuracy-Time-Memory can be given here
- Advanced parameters: Select Grids, Loss-functions, ...
- Low-Level configuration of the 3 phases




