#!/bin/bash
 
# Copyright 2017 Ingo Steinwart
#
# This file is part of liquidCluster.
#
# liquidCluster is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.
#
# liquidCluster is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


  


# Determine where I am and where the data might be.

SML_SCRIPTS_DIR=`dirname $0`
source $SML_SCRIPTS_DIR/auxiliary/get-directories.sh


#  Set constants


CLUSTER_TRUTH=0
CLUSTER_HIST=1
CLUSTER_KERN=3

# Set default values

source $SML_SCRIPTS_DIR/set-default-values.sh

THREADS=0
CLUSTER_ALGO=$CLUSTER_KERN
CLUSTER_HIER=-1
TAU_FACTOR=2
EPS_FACTOR=3
HIERARCHICAL_DEPTH=-1
PARTITIONER=2

# Display help message

if [[ -z "$1" ]]
then
	echo
	echo "cluster.sh is used in the following way:"
	echo
	echo "cluster.sh <base_filename> [<display>] [<threads>] [<cluster_algo>] [<grid_choice>] [<hierarchical_depth>] [<tau_factor>] [<eps_factor>]" 
	echo 
	echo Default values are:
	echo
	echo display:"               "$DISPLAY
	echo threads:"               "$THREADS
	echo cluster_algo:"          "$CLUSTER_KERN
	echo grid_choice:"           "$GRID_CHOICE
	echo hierarchical_depth:"   "$HIERARCHICAL_DEPTH
	echo tau_factor:"            "$TAU_FACTOR
	echo eps_factor:"            "$EPS_FACTOR
	echo
	exit
fi


# Copy command line arguments into variables 


BASE_FILENAME="$1"
if ! [[ -z "$2" ]]
then
	DISPLAY_OPTS="-d ""$2"
fi

if ! [[ -z "$3" ]]
then
	THREADS="$3"
fi

LABEL_POS=0
CLUSTER=TRUE
CLUSTER_DENS=FALSE
if ! [[ -z "$4" ]]
then
	CLUSTER_ALGO="$4"
	if [[ $CLUSTER_ALGO = $CLUSTER_TRUTH ]]
	then
		CLUSTER_DENS=TRUE
		WEIGHT_POS=1
	fi
fi

if ! [[ -z "$5" ]]
then
	GRID_CHOICE="$5"
fi

if ! [[ -z "$6" ]]
then
	HIERARCHICAL_DEPTH="$6"
fi

if ! [[ -z "$7" ]]
then
	TAU_FACTOR="$7"
fi

if ! [[ -z "$8" ]]
then
	EPS_FACTOR="$8"
fi





# Set delta grid

if [[ $GRID_CHOICE = 0 ]]
then
	MIN_DELTA=0.75
	MAX_DELTA=0.25
	DELTA_STEPS=50
	DELTA_SCALE=1
elif [[ $GRID_CHOICE = 1 ]]
then
	MIN_DELTA=0.25
	MAX_DELTA=0.3
	DELTA_STEPS=100
	DELTA_SCALE=1
elif [[ $GRID_CHOICE = 2 ]]
then
	MIN_DELTA=0.02
	MAX_DELTA=0.075
	DELTA_STEPS=50
	DELTA_SCALE=0
elif [[ $GRID_CHOICE = 3 ]]
then
	MIN_DELTA=0.01
	MAX_DELTA=0.1
	DELTA_STEPS=100
	DELTA_SCALE=0
else
	MIN_DELTA=0.04
	MAX_DELTA=
	DELTA_STEPS=
	DELTA_SCALE=
fi


# Get file names

ALLOW_NLA=TRUE
source $SML_SCRIPTS_DIR/get-file-names.sh



# Cluster data 

THREAD_OPTS="-T "$THREADS
HIER_OPTS="-H "$HIERARCHICAL_DEPTH
ALGO_OPTS="-E "$CLUSTER_ALGO

PREC_OPTS="-t "$TAU_FACTOR" -e "$EPS_FACTOR
DELTA_OPTS="-D "$MIN_DELTA" "$MAX_DELTA" "$DELTA_STEPS" "$DELTA_SCALE
PARTITIONER_OPTS="-P "$PARTITIONER

if ! [[ -z $RANDOM_SEED ]]
then
	RANDOM_SEED_OPT="-r "$RANDOM_SEED
fi

if [[ $NO_TEST_FILE = TRUE ]]
then
	TEST_FILENAME=
	PREDICT_FILENAME=
elif [[ $CLUSTER_DENS = TRUE ]]
then
	LABEL_POS=0
	WEIGHT_POS=0
	FILE_FORMAT=" +l "$LABEL_POS" +w "$WEIGHT_POS" +i "$ID_POS" +g "$GROUP_ID_POS" "
	TEST_FILENAME="$SML_DATA_DIR/$BASE_FILENAME.test.csv "$FILE_FORMAT
	LABEL_POS=1
	WEIGHT_POS=0
	FILE_FORMAT=" +l "$LABEL_POS" +w "$WEIGHT_POS" +i "$ID_POS" +g "$GROUP_ID_POS" "
	PREDICT_FILENAME="$SML_DATA_DIR/$BASE_FILENAME.truth.csv "$FILE_FORMAT
fi
$SML_BIN_DIR/dbc  $DISPLAY_OPTS $THREAD_OPTS $ALGO_OPTS $PARTITIONER_OPTS $DELTA_OPTS  $PREC_OPTS $HIER_OPTS  $RANDOM_SEED_OPT -s 0 1 $TRAIN_FILENAME $RES_TEST_FILENAME $TEST_FILENAME $PREDICT_FILENAME 



