// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.




#include "sources/shared/system_support/os_specifics.h"
#include "sources/shared/basic_functions/flush_print.h"
#include "sources/shared/basic_types/dataset_info.h"
#include "sources/shared/system_support/timing.h"
#include "sources/shared/kernel/histogram.h"

#include "sources/cluster/solver/density_cluster_manager.h" 
#include "sources/cluster/command_line/command_line_cluster.h"

#include "sources/shared/basic_functions/random_generator.h"

//**********************************************************************************************************************************



int main(int argc, char **argv)
{
	Tcommand_line_parser_cluster command_line_parser;
	Tdataset dataset;
	Tdataset test_set;
	Tdataset_info dataset_info;
	Tdensity_cluster_manager cluster_manager;
	unsigned i;
	Tcluster_compare best_cluster_compare;
	Tcluster_info cluster_info;
	vector <double> cluster_affiliation;
	vector <double> test_set_cluster_affiliation;
	vector <unsigned> cluster_sizes;
	vector <double> errors_on_clusters;
	double read_time;
	double write_time;
	double full_time;
	

// Read command line

	full_time = get_wall_time_difference();
	
	command_line_parser.setup(argc, argv);
	command_line_parser.parse();


// Load data set

	read_time = get_process_time_difference();
	dataset.read_from_file(command_line_parser.train_file_format);
	read_time = get_process_time_difference(read_time);
	

// Cluster train set
		
	
	cluster_manager.load(dataset, command_line_parser.cluster_control, cluster_info, command_line_parser.get_parallel_control());
	cluster_affiliation = cluster_manager.get_best_cluster_tree(best_cluster_compare, cluster_info);

	
// Write cluster numbers into label of data set and write everything to file
	
	cluster_info.test_time = get_wall_time_difference(cluster_info.test_time);
	for (i=0; i<dataset.size(); i++)
		dataset.set_label_of_sample(i, cluster_affiliation[i]);
	cluster_info.test_time = get_wall_time_difference(cluster_info.test_time);
	
	write_time = get_process_time_difference();
	dataset.write_to_file(command_line_parser.result_file_format);
	write_time = get_process_time_difference(write_time);
	

	dataset_info = Tdataset_info(dataset, true);
	for (i=0; i<dataset_info.label_list.size(); i++)
		flush_info(INFO_1, "\nLabel %2d  occurs %d times (%1.4f)", dataset_info.label_list[i], dataset_info.label_count[i], float(dataset_info.label_count[i])/float(dataset_info.size));

    
// Apply clustering to optional test set
	
	if (command_line_parser.test_file_format.filename.size() > 0)
	{
		read_time = get_process_time_difference();
		test_set.read_from_file(command_line_parser.test_file_format);
		read_time = get_process_time_difference(read_time);

		cluster_affiliation = cluster_manager.assign_cluster_to_test_set(test_set, cluster_info);

		cluster_info.test_time = get_wall_time_difference(cluster_info.test_time);
		if (test_set.is_classification_data() == true)
		{
			test_set_cluster_affiliation = test_set.get_labels();
			errors_on_clusters = cluster_manager.compute_test_error(cluster_affiliation, test_set_cluster_affiliation, cluster_sizes);
		}

		for (i=0; i<test_set.size(); i++)
			test_set.set_label_of_sample(i, cluster_affiliation[i]);
		cluster_info.test_time = get_wall_time_difference(cluster_info.test_time);
		
		write_time = get_process_time_difference();
		test_set.write_to_file(command_line_parser.test_result_file_format);
		write_time = get_process_time_difference(write_time);
		
		flush_info(INFO_1, "\n\nStatistics for the additional test set:");
		dataset_info = Tdataset_info(test_set, true);
		for (i=0; i<dataset_info.label_list.size(); i++)
			flush_info(INFO_1, "\nLabel %2d  occurs %d times (%1.4f)", dataset_info.label_list[i], dataset_info.label_count[i], float(dataset_info.label_count[i])/float(dataset_info.size));
		
	}
	
	
	// Clean up

 	full_time = get_wall_time_difference(full_time);
	cluster_info.display(TRAIN_INFO_DISPLAY_FORMAT_TIMINGS_FULL, INFO_1);
	flush_info(INFO_1,"\n\n%4.2f seconds used to run cluster.", full_time);
	flush_info(INFO_1,"\n%4.2f seconds used for read from file operations.", read_time);
	flush_info(INFO_1,"\n%4.2f seconds used for write to file operations.", write_time);
	
	flush_info(INFO_1,"\n\n");
}


