// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (CLUSTER_AFFILIATION_CLASS_H) 
 #define CLUSTER_AFFILIATION_CLASS_H


 
// #include "sources/shared/basic_types/dataset_info.h"
#include "sources/shared/basic_types/my_unordered_map_and_set.h"


#include "sources/cluster/solver/cluster_tree.h" 




#include <list>
#include <vector>



//**********************************************************************************************************************************

 
typedef my_unordered_set <double> cluster_number_store_type;
typedef my_unordered_map <double, Tpeak> cluster_maxima_store_type;
typedef my_unordered_map <double, list <unsigned> > cluster_store_type;


 
//**********************************************************************************************************************************


class Tcluster_affiliation
{
	public:
		Tcluster_affiliation();

		void clear();
		void reserve(unsigned size);
		void new_level_set();
		
		void assign_sample_to_cluster(unsigned index, double cluster_number, double rho);
		void merge_clusters(double cluster_number_1, double cluster_number_2, double& new_cluster_number);
		void change_cluster_number(double old_cluster_number, double new_cluster_number);
		
		double get_largest_inactive_maximum();
		
		inline unsigned get_unassigned_sample() const;
		inline double get_cluster_number(unsigned index) const;
		inline unsigned get_size_of_cluster(double cluster_number);
		inline Tpeak get_maximum_of_cluster(double cluster_number);
		inline unsigned get_number_of_assigned_samples() const;
		inline vector <double> get_cluster_affiliations() const;
		inline unsigned get_number_of_samples_with_new_cluster_number() const;
		inline void get_sample_with_new_cluster_number(unsigned& index, double& cluster_number);

		inline cluster_number_store_type get_current_cluster_numbers() const;
		inline cluster_number_store_type get_current_active_cluster_numbers() const;
		
		inline double get_largest_cluster_number() const;
		inline double get_smallest_cluster_number() const;
		inline unsigned get_number_of_current_active_cluster() const;
	
		void display();
		
		
	private:
		void delete_cluster(double cluster_number);
		void create_new_cluster(double cluster_number);
		void move_cluster_numbers(double source_cluster_number, double target_cluster_number);
		
		
		vector <double> cluster_affiliations;
		cluster_store_type clusters;
		cluster_maxima_store_type cluster_maxima;
		
		cluster_number_store_type current_cluster_numbers;
		my_unordered_set <unsigned> unassigned_samples;
		my_unordered_set <unsigned> samples_with_new_cluster_number;
		
		double min_cluster_number;
		double max_cluster_number;
}; 
 
 
 
 
//**********************************************************************************************************************************


#include "sources/cluster/solver/cluster_affiliation.ins.cpp" 


#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/cluster_affiliation.cpp" 
#endif
 
 
#endif
