// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.







//**********************************************************************************************************************************

inline double Tcluster_affiliation::get_largest_cluster_number() const
{
	return max_cluster_number;
}
//**********************************************************************************************************************************

inline double Tcluster_affiliation::get_smallest_cluster_number() const
{
	return min_cluster_number;
}



//**********************************************************************************************************************************


inline cluster_number_store_type Tcluster_affiliation::get_current_cluster_numbers() const
{
	return current_cluster_numbers;
}



//**********************************************************************************************************************************


inline vector <double> Tcluster_affiliation::get_cluster_affiliations() const
{
	return cluster_affiliations;
}

//**********************************************************************************************************************************


inline double Tcluster_affiliation::get_cluster_number(unsigned index) const
{
	if (index >= cluster_affiliations.size())
		flush_exit(ERROR_DATA_MISMATCH, "Cannot access sample with number %d in a cluster affiliation object of size %d.", index, cluster_affiliations.size());
	
	return cluster_affiliations[index];
}



//**********************************************************************************************************************************

inline unsigned Tcluster_affiliation::get_number_of_assigned_samples() const
{
	return unsigned(int(cluster_affiliations.size()) - int(unassigned_samples.size()));
}

//**********************************************************************************************************************************

inline unsigned Tcluster_affiliation::get_unassigned_sample() const
{
  if (unassigned_samples.size() == 0)
		flush_exit(ERROR_DATA_MISMATCH, "Cannot return unassigned sample since all samples are assigned.");
	
	return *(unassigned_samples.begin());
}




//**********************************************************************************************************************************

inline unsigned Tcluster_affiliation::get_size_of_cluster(double cluster_number)
{
	cluster_store_type::iterator jt;
	
	
	if (cluster_number == 0.0)
		return unassigned_samples.size();
	else
	{
		jt = clusters.find(cluster_number);
		if (jt == clusters.end())
			return 0;
		else
			return jt->second.size();
	}
}


//**********************************************************************************************************************************


inline Tpeak Tcluster_affiliation::get_maximum_of_cluster(double cluster_number)
{
	cluster_maxima_store_type::iterator jt;
	
	
	if (cluster_number == 0.0)
		flush_exit(ERROR_DATA_MISMATCH, "Cluster 0 does not have a maximum.");
	
	jt = cluster_maxima.find(cluster_number);
	if (jt == cluster_maxima.end())
		flush_exit(ERROR_DATA_MISMATCH, "Cannot return maximum since the cluster with number %2.4f does not exist.", cluster_number);

	jt->second.cluster_number = cluster_number;
	
	return jt->second;
}




//**********************************************************************************************************************************


inline cluster_number_store_type Tcluster_affiliation::get_current_active_cluster_numbers() const
{
	cluster_number_store_type cluster_numbers_tmp;
	
	for (auto jt = current_cluster_numbers.begin(); jt != current_cluster_numbers.end(); ++jt) 
		if (*jt > 0.0)
			cluster_numbers_tmp.insert(*jt);

	return cluster_numbers_tmp;
}


//**********************************************************************************************************************************


inline unsigned Tcluster_affiliation::get_number_of_current_active_cluster() const
{
	unsigned count;
	
	count = 0;
	for (auto jt = current_cluster_numbers.begin(); jt != current_cluster_numbers.end(); ++jt) 
		if (*jt > 0.0)
			count++;

	return count;
}


//**********************************************************************************************************************************


inline unsigned Tcluster_affiliation::get_number_of_samples_with_new_cluster_number() const
{
	return samples_with_new_cluster_number.size();
}


//**********************************************************************************************************************************



inline void Tcluster_affiliation::get_sample_with_new_cluster_number(unsigned& index, double& cluster_number)
{
	std::my_unordered_set <unsigned>::iterator it;
	
	
	if (samples_with_new_cluster_number.size() == 0)
		cluster_number = -1.0;
	else
	{
		it = samples_with_new_cluster_number.begin();
		index = *it;
		cluster_number = cluster_affiliations[index];
		samples_with_new_cluster_number.erase(index);
	}
}












