// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (CLUSTER_CONTROL_CLASS_H) 
	#define CLUSTER_CONTROL_CLASS_H

	
	
	
#include <vector>
using namespace std;

//**********************************************************************************************************************************

	
enum DENSITY_ESTIMATOR_TYPES {DENSITY_TRUTH, DENSITY_HISTOGRAMM, DENSITY_CONSTANT_SUP_KERNEL, DENSITY_CONSTANT_EUCLIDEAN_KERNEL, DENSITY_TRIANGULAR_KERNEL, DENSITY_EPANECHNIKOV, DENSITY_BIWEIGHT_KERNEL, DENSITY_TRIWEIGHT_KERNEL, DENSITY_ALGORITHMS_MAX, DENSITY_GENERIC_KERNEL};
const int UNLIMITED_CLUSTER_DEPTH = -1;


enum DOMAIN_PARTITIONER_TYPES {NO_PARTITIONER, COVER_TREE, ADAPTIVE_PARTITIONER, DOMAIN_PARTITIONER_TYPES_MAX};


//**********************************************************************************************************************************


class Tdensity_control
{
	public:
		Tdensity_control();

		unsigned get_density_estimator_category() const;
		unsigned get_metric_of_estimator() const;
		double get_estimator_integral(unsigned dim) const;
		double unit_ball_volume(unsigned dim) const;
		double kernel_function(double distance) const;
		void scale_deltas(unsigned data_size, unsigned dim);
		
		
		unsigned density_estimator_type;
		unsigned domain_partitioner_type;
		
		double min_delta;
		double max_delta;
		unsigned delta_steps;
		bool scale_delta_range;
		double max_delta_neighborhood;
		
		vector <double> deltas_scaled;
		vector <double> deltas;
};





//**********************************************************************************************************************************


class Tcluster_control: public Tdensity_control
{
	public:
		Tcluster_control();

		double get_tau_from_delta(double delta) const;

		
		double tau_factor;
		double eps_factor;
		int hierarchical_depth;
		
		unsigned cluster_compare_method;
		double rho_compare_tolerance_factor;
		unsigned test_set_size;
		int random_seed;
};



//**********************************************************************************************************************************


#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/cluster_control.cpp"
#endif










#endif

