// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (CLUSTER_TREE_CLASS_H) 
 #define CLUSTER_TREE_CLASS_H


 

#include "sources/shared/basic_types/my_unordered_map_and_set.h"


#include "sources/cluster/solver/tree.h"





//**********************************************************************************************************************************


struct Tpeak
{
	double height;
	double cluster_number;
	my_unordered_set <unsigned> indices;
	
	bool operator == (const Tpeak& peak) const;
};

//**********************************************************************************************************************************

struct Tcluster
{
	Tcluster();
	Tcluster(const Tcluster& cluster);
	
	double rho;
	double rho_star;
	double cluster_number;
	double parent_cluster_number;
	double width;
	unsigned size;
	unsigned max_number_of_core_samples;
	vector <Tpeak> peaks;
	vector <unsigned> core_samples;
	
	void clear();
	void merge(const Tcluster& cluster); 
	void merge(const Tcluster& cluster_1, const Tcluster& cluster_2); 
	void display(unsigned info_level);
	
	Tcluster& operator = (const Tcluster& cluster);
	void copy(const Tcluster& source_cluster);
};


//**********************************************************************************************************************************

typedef Tforest <Tcluster> Tcluster_tree;
typedef Ttree_node <Tcluster> Tcluster_tree_node;


//**********************************************************************************************************************************


#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/cluster_tree.cpp" 
#endif



#endif
