// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (DENSITY_CLUSTER_MANAGER_H) 
 #define DENSITY_CLUSTER_MANAGER_H


#include "sources/shared/system_support/compiler_specifics.h"
#include "sources/shared/system_support/thread_manager_active.h"
 
#include "sources/shared/basic_types/dataset.h" 
#include "sources/shared/kernel/histogram.h"



#include "sources/cluster/solver/cluster_control.h"
#include "sources/cluster/training_validation/cluster_info.h"

#include "sources/cluster/solver/partitioner.h" 
#include "sources/cluster/solver/cluster_tree.h"
#include "sources/cluster/solver/cluster_compare.h" 
#include "sources/cluster/solver/density_cluster_tree.h" 






//**********************************************************************************************************************************


class Tdensity_cluster_manager: protected Tthread_manager_active
{
	public:
		Tdensity_cluster_manager();
		~Tdensity_cluster_manager();
		
		void clear();
		void load(const Tdataset& dataset, const Tcluster_control& cluster_control, Tcluster_info& cluster_info, Tparallel_control parallel_ctrl);

		
		vector <double> get_best_cluster_tree(Tcluster_compare& best_cluster_compare, Tcluster_info& cluster_info);
				
		vector <double> assign_cluster_to_test_set(const Tdataset& test_set, Tcluster_info& cluster_info);
		vector <double> compute_test_error(const vector <double>& cluster_affiliation, const vector <double>& true_cluster_affiliation, vector <unsigned>& true_cluster_sizes);

		
	private:
		void thread_entry();

		void neighborhood_build(double max_distance, Tcluster_info& cluster_info);
		void neighborhood_build_threaded();
		
		void neighborhood_sort(Tcluster_info& cluster_info);
		void neighborhood_sort_threaded();
		
		void compute_all_cluster_trees(Tcluster_compare& best_cluster_compare, Tcluster_info& cluster_info);
		void compute_all_cluster_trees_threaded();

		void compute_excess_mass_etc(Tcluster_info& cluster_info);
		void compute_excess_mass_etc_threaded();
		
		void assign_cluster_to_test_set_threaded();
		
		void get_level_set(unsigned candidate_index);
		
		void find_higher_rho_stars(const vector <double>& cluster_affiliation, double rho_star, double cluster_number, double& best_delta, Tcluster_tree_node*& best_merge, Tcluster_info& cluster_info);
		bool peak_in_cluster(Tpeak* peak, const vector <double>& cluster_affiliation, double cluster_number, double rho_star);
		unsigned count_peaks_in_cluster(vector <Tpeak>* peaks, const vector <double>& cluster_affiliation, double cluster_number, double rho_star);
		
		
		void get_core_sample_affiliation_from_forest(Tcluster_tree* cluster_tree, vector <double>& core_cluster_affiliation);
		
		void assign_core_samples_of_cluster(Ttree_node <Tcluster>* cluster_node, Ttree_node <Tcluster>* target_cluster_node, vector <double>& core_cluster_affiliation, double parent_cluster_number, double cluster_number, double delta, const vector <double> cluster_affiliation, Tcluster_info& cluster_info);
		
		void change_cluster_number_according_to_labels(vector <double>& cluster_affiliation);
		vector <double> get_cluster_numbers_of_clustering(const vector <double>& cluster_affiliation);
		
		void show_clustering_statistics(const vector <double>& cluster_affiliation);
		void check_tree_consistency(Tcluster_tree* cluster_tree);

		
		void identify_best_cluster(unsigned& index_of_best_cluster, Tcluster_info& cluster_info);
		
		
		unsigned thread_task;
		double neighborhood_tau;
		Tcluster_control cluster_control;

		Tdataset dataset;
		Tdataset test_set;
		
		bool use_cover_tree;
		Tdomain_partitioner domain_partitioner;
		Tneighborhood_lists neighborhood_lists;
		
		vector <unsigned> level_set;
		Tsubset_info random_level_subset;
		
		my_unordered_map <double, Tcluster_tree > cluster_tree;
		my_unordered_map <double, Tdensity_cluster_tree> density_cluster_tree_estimator;
		
		Tcluster_tree final_cluster_tree;
		my_unordered_map <double, double> cluster_trafo;

		Ttree_node <Tcluster>* current_node_for_all_threads;
		vector <double> test_affiliation;
		vector <double> new_test_affiliation;
		
		list <Tcluster_info> cluster_infos;
		vector <Tcluster_info> cluster_info_of_thread;
		
		vector <unsigned> candidate_runs;
		vector <Texcess_mass> excess_mass_of_thread;
		
		vector <Tcluster_compare> cluster_compare_list;
		vector <Tcluster_compare> cluster_compare_candidates;
};



//**********************************************************************************************************************************


#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/density_cluster_manager.cpp" 
#endif
	
#endif

