// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (DENSITY_CLUSTER_TREE_H) 
 #define DENSITY_CLUSTER_TREE_H


 
 

#include "sources/shared/kernel/histogram.h"


#include "sources/cluster/training_validation/cluster_info.h"
#include "sources/cluster/solver/cluster_affiliation.h" 

	
#include "sources/cluster/solver/tree.h"
#include "sources/cluster/solver/cluster_tree.h"

#include "sources/cluster/solver/partitioner.h" 

#include "sources/cluster/solver/density_estimator.h" 
 

//**********************************************************************************************************************************
 
 
 
 
class Tdensity_cluster_tree: public Tdensity_estimator
{
	public:
		Tdensity_cluster_tree();
		~Tdensity_cluster_tree();
		
		void clear();
		void load(const Tdataset& dataset, const Tdataset& random_set, const Tcluster_control& cluster_control, Tcluster_info& cluster_info);

		void compute_cluster_tree(double delta, Tcluster_tree& new_cluster_tree, Tcluster_info& cluster_info, double& rho_star, list <Tcluster_info>& all_cluster_infos);


	private:
		void get_level_set(double rho);
		void provide_density_estimate(double delta, Tcluster_info& cluster_info);
		
		void insert_root_in_cluster_tree(double cluster_number, double rho);
		void get_cluster(double rho, Tcluster_info& cluster_info);
		void get_next_merge(double min_rho, Tcluster_info& cluster_info, unsigned& number_of_rho_steps);

		double get_eps_uncertainty() const;
		
		double get_start_rho() const;
		double get_max_rho_precision() const;
		double get_rho_step(double old_rho) const;
		
		void convert_mid_points_to_peak_samples();


		double old_rho;
	
		unsigned level_set_start;
		unsigned level_set_stop;
		vector <unsigned> density_ordering;
		
		Tcluster_tree cluster_tree;
		Tcluster_control cluster_control;
		list <Tcluster_info> cluster_infos;
		Tcluster_affiliation cluster_affiliations;
};
 
 
 
 
 
//**********************************************************************************************************************************
 
#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/density_cluster_tree.cpp" 
#endif

#endif
