// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (DENSITY_ESTIMATOR_H) 
 #define DENSITY_ESTIMATOR_H


#include "sources/shared/kernel/histogram.h"

#include "sources/cluster/solver/partitioner.h" 
#include "sources/cluster/solver/cluster_control.h"

#include "sources/cluster/training_validation/density_info.h"
 

 
//**********************************************************************************************************************************
 
struct Texcess_mass
{
	double density_integral;
	double constant_integral;
};
 
 
//**********************************************************************************************************************************
 
 
class Tdensity_estimator
{
	public:
		Tdensity_estimator();
		~Tdensity_estimator();
		
	
		void clear();
		void set_info_level(unsigned info_level);
		void load(const Tdataset& dataset, const Tdataset& random_set, const Tdensity_control& density_control, bool set_working_set);
		void prepare_test_set(unsigned test_set_size, const Tsubset_info& level_set_core_samples, Tdensity_info& density_info);
		void construct_test_set(unsigned start_index, unsigned stop_index, double delta, const Tsubset_info& random_level_subset, Tdensity_info& density_info);
		void import_spatial_information(Tdomain_partitioner* domain_partitioner, Tneighborhood_lists* neighborhood_lists);
		
		vector <double> get_density_estimate(double delta, Tdensity_info& density_info);
		Texcess_mass get_excess_mass(unsigned start_index, unsigned stop_index, Tdensity_info& density_info);
		
		
		vector <unsigned> reassign_core_samples(const Tsubset_info& core_samples, vector <double>& core_sample_affiliation, double active_affiliation_number, double new_affiliation_number, const vector <double> cluster_affiliation);
		
		vector <double> expand_from_core_to_train_samples(double sigma, double active_affiliation_number, double new_affiliation_number, Tdensity_info& density_info, const vector <double>& core_samples_affiliation, const vector <double>& current_affiliation);
		
		void expand_from_core_to_test_samples(unsigned start_index, unsigned stop_index, double sigma, double active_affiliation_number, double new_affiliation_number, vector <double>& new_affiliation, const Tdataset& test_set, const vector <double>& current_affiliation, const vector <unsigned>& current_core_samples, const vector <double>& core_samples_affiliation);
		

		inline double get_current_delta() const;
		
	protected:
		void compute_density_estimate(double delta, Tdensity_info& density_info);
		double get_min_density() const;
		double get_max_density() const;
		double get_scale_factor(double delta, unsigned size) const;
		
		
		unsigned info_level;
		
		unsigned dim;
		unsigned true_size;
		unsigned working_set_size;
		
		Tdataset dataset;
		Tdataset working_set;
		
		Tdataset_info dataset_info;
		bool dataset_info_computed;
		
		
		vector <double> density_estimate;
		
		Tdensity_control density_control;
		
		Thistogram histogram;
		Tdomain_partitioner* domain_partitioner;
		Tneighborhood_lists* neighborhood_lists;
		
	private:
		void density_estimate_histogram(double& density_time, double& histogram_time);
		void density_estimate_kernel(const vector <vector <double> >& lists_of_distances, double& density_time);
		void density_estimate_true(double& density_time);
		
		Texcess_mass excess_mass_histogram(unsigned start_index, unsigned stop_index, Tdensity_info& density_info);
		Texcess_mass excess_mass_kernel(unsigned start_index, unsigned stop_index, Tdensity_info& density_info);
		Texcess_mass excess_mass_true(unsigned start_index, unsigned stop_index, Tdensity_info& density_info);
		
		
		void expand_from_core_histogram(unsigned start_index, unsigned stop_index, double sigma, double new_affiliation_number, vector <double>& new_affiliation, const Tdataset& sample_set, const vector <double>& core_samples_affiliation, const vector <double>& current_affiliation);
		
		void expand_from_core_to_test_kernel(unsigned start_index, unsigned stop_index, double sigma, double active_affiliation_number, double new_affiliation_number, vector <double>& new_affiliation, const Tdataset& test_set, const vector <double>& current_affiliation, const vector <unsigned>& current_core_samples, const vector <double>& core_samples_affiliation);
		
		void empty_density_estimate();
		void reset_density();
		
		void internal_neighborhood_build(double max_distance, double& tau_neighbor_build_time, double& tau_neighbor_sort_time);
		bool is_valid_neighbor(double own_affiliation, double neighbor_affiliation) const;
		
		
		
		double delta;
		double max_density;
		double min_density;
		
		vector <Tsample> test_set;
// 		Tdataset test_set;
		Tdataset random_set;
		vector <unsigned> level_set;
		
		Tdomain_partitioner internal_domain_partitioner;
		Tneighborhood_lists internal_neighborhood_lists;
};
 
 
//**********************************************************************************************************************************


#include "sources/cluster/solver/density_estimator.ins.cpp" 

#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/density_estimator.cpp" 
#endif



#endif
