// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (DOMIAIN_PARTITIONER_H) 
 #define DOMIAIN_PARTITIONER_H



#include "sources/shared/basic_types/dataset.h"
#include "sources/shared/training_validation/working_set_control.h"

 
#include "sources/cluster/solver/tree.h"




//**********************************************************************************************************************************



class Tcover_cell
{
	public:
		void clear();
		void copy(const Tcover_cell& source_cell);

		
		Tsample center_sample;
		unsigned center_index;
		Tsubset_info member_indices;
		
		double radius;
		int cell_number;
};


//**********************************************************************************************************************************


struct Tneighborhood_lists
{
	vector <Tsubset_info> neighbors;
	vector <vector <double> > distances;
};


//**********************************************************************************************************************************


typedef Ttree <Tcover_cell> Tcover_tree;
typedef Ttree_node <Tcover_cell> Tcover_tree_node;


//**********************************************************************************************************************************

const double TRUE_MASK_ENTRY = 1.0;
const double FALSE_MASK_ENTRY = -1.0;

//**********************************************************************************************************************************


class Tdomain_partitioner
{
	public:
		Tdomain_partitioner();
		~Tdomain_partitioner();
		
		void clear();
		void clear_tree();
		void load(const Tdataset& dataset);
		
		bool is_cover_tree_built() const;
		void build_cover_tree(const Tworking_set_control& working_set_control, unsigned metric);
		
		void get_tau_neighborhood(Tsample* sample, double tau, Tsubset_info& subset_info, vector <double>& distances, unsigned metric, const vector <double>& mask) const;

		void build_neighborhood_lists(unsigned start_index, unsigned stop_index, double tau, unsigned metric, Tneighborhood_lists& neighborhood_lists);
		
		
	protected:
		Tcover_tree cover_tree;
		
	private:
		void build_cover_tree_recursively(const Tdataset& current_working_set, Tsubset_info current_subset_info, Tcover_tree_node* current_tree_node, unsigned current_depth, unsigned NNs, unsigned metric);
		
		
		bool cover_tree_built;
		unsigned metric;
		unsigned partitioner_type;
		Tdataset dataset;
		Tworking_set_control working_set_control;
};


//**********************************************************************************************************************************

#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/solver/partitioner.cpp" 
#endif



#endif
