// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (TREE_H) 
 #define TREE_H
 
 
 
#include <map>
#include <vector>
#include <stack>
#include <queue>
using namespace std;
 
 
//**********************************************************************************************************************************


template <class Tcontainer_type> class Ttree_node: public Tcontainer_type
{
	public:
		Ttree_node();
		Ttree_node(const Ttree_node<Tcontainer_type>& tree_node);
		~Ttree_node();
		
		void clear();
		Ttree_node<Tcontainer_type>& operator = (const Ttree_node<Tcontainer_type>& tree_node);
		void display(unsigned info_level);
		
	
		unsigned depth;
		Ttree_node* parent_node;
		vector <Ttree_node*> child_nodes;
		
	protected:
		void copy(const Ttree_node& source_node);
};



//**********************************************************************************************************************************


template <class Tcontainer_type> class Ttree
{
	public:
		Ttree();
		Ttree(const Ttree <Tcontainer_type>& tree);
		~Ttree();

		void clear();
		void display( unsigned info_level);
		unsigned nodes();
		unsigned leafs();
		
		Ttree_node <Tcontainer_type>* start();
		Ttree_node <Tcontainer_type>* next();
		
		Ttree <Tcontainer_type>& operator = (const Ttree <Tcontainer_type>& tree);

		
		
		Ttree_node <Tcontainer_type> root_node;

	private:
		void copy(const Ttree <Tcontainer_type>& source_tree);
		
		
		stack <Ttree_node <Tcontainer_type>*> stack_of_nodes;
		queue <Ttree_node <Tcontainer_type>*> queue_of_nodes;
};






//**********************************************************************************************************************************


template <class Tcontainer_type> class Tforest
{
	public:
		Tforest();
		Tforest(const Tforest <Tcontainer_type>& forest);
		~Tforest();
		
		void clear();
		void display(unsigned info_level);
		
		
		void new_root(double target_key);
		void merge_roots(double target_key, double source_key);
		void merge_roots(double target_key, double source_key_1, double source_key_2);
		void unite_roots(double target_key, double source_key);
		void unite_roots(double target_key, double source_key_1, double source_key_2);
		bool root_exists(double key);
		
		Ttree_node <Tcontainer_type>* start();
		Ttree_node <Tcontainer_type>* start(double key);
		Ttree_node <Tcontainer_type>* next();
		
		Tforest <Tcontainer_type>& operator = (const Tforest <Tcontainer_type>& forest);
		
		
		map < double, Ttree_node <Tcontainer_type> > root_nodes;
		
	protected:
		void check_key(double key);
		
	private:
		void copy(const Tforest <Tcontainer_type>& source_forest);
		void clear_recursively(Ttree_node <Tcontainer_type>* current_tree_node);
		
	
		stack <Ttree_node <Tcontainer_type>*> stack_of_nodes;
		queue <Ttree_node <Tcontainer_type>*> queue_of_nodes;
		
		
		bool delete_queue;
};
 
 
 

//**********************************************************************************************************************************


#include "sources/cluster/solver/tree.ins.cpp"


//**********************************************************************************************************************************


 
 
#endif
