// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (CLUSTER_INFO_H) 
	#define CLUSTER_INFO_H


	
#include "sources/cluster/training_validation/density_info.h"
	
	

//**********************************************************************************************************************************
	
const double NO_RHO = -1.0;
	
//**********************************************************************************************************************************

class Tcluster_info: public Tdensity_info
{
	public:
		Tcluster_info();
		Tcluster_info(const Tdensity_info& density_info);
		Tcluster_info(const Tcluster_info& cluster_info);
		
		void clear();
		void clear_time();
		void copy_hyper_parameters(const Tcluster_info& cluster_info);
		void copy_cluster_results(const Tcluster_info& cluster_info);

		virtual double get_full_time() const;
		
		Tcluster_info operator + (const Tcluster_info& cluster_info);
		Tcluster_info operator = (const Tcluster_info& cluster_info);


		double tau;
		double eps;
		double rho_step;
		double min_rho_step;
		double rho_start;
		double rho_stop;
		double rho;
		double rho_star;

		bool changed_level_set;
		unsigned number_of_clusters;
		unsigned number_of_merges;
		unsigned number_of_new_merges;
		unsigned number_of_peaks;
		unsigned number_of_new_peaks;
		unsigned number_of_assigned_samples;
		unsigned number_of_unassigned_samples;
		unsigned number_of_newly_assigned_samples;
		unsigned number_of_samples_with_new_cluster_number;
		
		double level_set_compute_time;
		double cluster_search_time;
		double cluster_connect_time;
		double cluster_expand_time;
		double cluster_admin_time;
		double cluster_tree_build_time;
		
	
	protected:
		void copy(const Tcluster_info& cluster_info);
		
		virtual string displaystring_detailed(unsigned display_mode) const;
		virtual string displaystring_train_time(unsigned display_mode) const;
};


//**********************************************************************************************************************************


Tcluster_info max(const Tcluster_info& cluster_info_1, const Tcluster_info& cluster_info_2);


//**********************************************************************************************************************************


#ifndef COMPILE_SEPERATELY__
	#include "sources/cluster/training_validation/cluster_info.cpp"
#endif


#endif
