

source("filename_scripts.r")
source("art_data.r")


# -------------------------------------------------------------------------------------------------------------------------


create_all_2d_datasets <- function(name, dim1=2, dim2=3, bgsize=200, maxsize=-1)
{
	create_2d_data(name, "train", dim1, dim2, maxsize)
	create_2d_data(name, "test", dim1, dim2, maxsize)
	create_background_data(name, bgsize)
}



# -------------------------------------------------------------------------------------------------------------------------


delete_all_2d_datasets <- function(name)
{
	delete_2d_data(name, "train")
	delete_2d_data(name, "test")
	delete_background_data(name, "test")
}


# -------------------------------------------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------------------------------------------


create_2d_data <- function(name, train_test, dim1=2, dim2=3, maxsize=-1)
{
	in_filename = get_data_filename(name, train_test)
	out_filename = get_full_2d_filename(name, train_test)
	print(in_filename)
	print(out_filename)

	dataset = read.csv(in_filename)

	if (maxsize == -1)
		maxsize = nrow(dataset)
	data = dataset[1:min(maxsize, nrow(dataset)), c(1, dim1, dim2)]

	write.table(data, file = out_filename, row.names = FALSE, col.names = FALSE, append = FALSE, sep = ", ", eol = "\n")
}

# -------------------------------------------------------------------------------------------------------------------------


delete_2d_data <- function(name, train_test)
{
	out_filename = get_full_2d_filename(name, train_test)

	system(paste("rm -f ", out_filename, sep=""))
}


# -------------------------------------------------------------------------------------------------------------------------


create_background_data <- function(name, size = 200)
{
	out_filename = get_full_bg_filename(name, "test")
	print(out_filename)
	
	data = create_grid_data(size)
	data[ ,1] = rep(1.0, nrow(data))
 
	write.table(data, file = out_filename, row.names = FALSE, col.names = FALSE, append = FALSE, sep = ", ", eol = "\n")
}



# -------------------------------------------------------------------------------------------------------------------------


create_grid_data <- function(size)
{
	square_size = size * size
	data = matrix(nrow = square_size,  ncol=3)
	data[ ,1] = rep(0.0, square_size)
	for (x in 1:size)
		for (y in 1:size)
		{
			data[(x - 1) * size + y, 2] = (y - 0.5) / size
			data[(x - 1) * size + y, 3] = (x - 0.5) / size
		}

	return(data)
}




# -------------------------------------------------------------------------------------------------------------------------

delete_background_data <- function(name, train_test)
{
	out_filename = get_full_bg_filename(name, "test")

	system(paste("rm -f ", out_filename, sep=""))
}


# -------------------------------------------------------------------------------------------------------------------------

restrict_data <- function(data, maxsize, dim1 = 2, dim2 = 3)
{
	data = data[sample(nrow(data)), ]
	
	data = data[data[ ,dim1]>=0.0, ]
	data = data[data[ ,dim1]<=1.0, ]
	data = data[data[ ,dim2]>=0.0, ]
	data = data[data[ ,dim2]<=1.0, ]
	
	data = data[1:min(maxsize, nrow(data)), ]
	
	return (data)
}


