library(graphics)
library(grDevices)





# ----------------------------------------------------------------------------------------------------------------------


plot_raw_datasets <- function(name, bg_include, ml_method="", maxsize=500000)
{
	train_data = read.table(get_full_2d_filename(name, "train"), sep=",")
	test_data = read.table(get_full_2d_filename(name, "test"), sep=",")
	if (bg_include == TRUE)
	{
		data_bg = read.table(get_data_filename(get_bg_name(name), "test"), sep=",")
		label_bg = read.table(get_result_filename(get_bg_name(name)), sep=",")
		data_bg = cbind(label_bg, data_bg[ ,2:3])
	}

	train_image_name = get_image_filename(name, "train", bg_include, ml_method)
	open_image(train_image_name)
	if (bg_include == TRUE)
		plot_binclass_bg_points(data_bg)
	plot_binclass_points(train_data)
	close_image()

	test_image_name = get_image_filename(name, "test", bg_include, ml_method)
	open_image(test_image_name)
	if (bg_include == TRUE)
	{
		plot_binclass_bg_points(data_bg)
		label_test = read.table(get_result_filename(get_xd_name(name, 2)), sep=",")
		test_data[ ,1] = label_test
	}
	plot_binclass_points(test_data)
	close_image()

	if (bg_include == FALSE)
	{
		all_image_name = get_image_filename(name, "all", bg_include, "")
		test_data[ ,1] = rep(0.0, nrow(test_data))
		open_image(all_image_name)
		plot_binclass_points(train_data)
		plot_binclass_points(test_data)
		close_image()
	}
}

# ----------------------------------------------------------------------------------------------------------------------


plot_histogram_mechanism <- function(name, size, number_of_colors = 255, include_boundary = TRUE)
{

	bg_data = read.table(get_full_bg_filename(get_toy_name(name, size), "train"), sep=",")
	bg_data[ ,1] = 1 + round((number_of_colors - 1) * bg_data[ ,1])
	size = sqrt(nrow(bg_data))

	
	train_image_name = get_image_filename(name, "train", TRUE, paste("hrm", size, number_of_colors, sep="-"))
	
	pal = colorRamp(c(redorange, bluegreen))((0:(number_of_colors - 1))/(number_of_colors-1))
	pal = rgb(pal/255)

	open_image(train_image_name)
	
	plot_bg(bg_data, pal)
	
	if (include_boundary == TRUE)
	{
		boundary_points = read.table(get_full_bg_filename(name, "boundary"), sep=",")
		lines(boundary_points[ ,1], boundary_points[ ,2], lwd=5)
	}

	dev.off()
}


# ----------------------------------------------------------------------------------------------------------------------

open_image <- function(full_image_name, pal = "white")
{
	jpeg(clean_name(full_image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)

	plot.new()
	par(mai=c(0.1, 0.1, 0.1, 0.1))
	plot.window(xlab="", ylab="", xlim = c(0, 1), ylim = c(0, 1))
	
	size = 10
	data = create_grid_data(size)
	plot_bg(data, pal)
}


# ----------------------------------------------------------------------------------------------------------------------

close_image <- function()
{
	dev.off()
}


# ----------------------------------------------------------------------------------------------------------------------

plot_binclass_points <- function(data, maxsize=200000, pointsize=c(1.5, 1.0, 1.5))
{
	data = restrict_data(data, maxsize)

	pal = c(bluegreen, "black", redorange)
# 	pointsize = c(1.5, 1.0, 1.5)
	pointform = c(20, 10, 20)

	points(data[ ,2:3], col=pal[2 + data[ ,1]], pch=pointform[2 + data[ ,1]], cex=pointsize[2 + data[ ,1]])
}


# ----------------------------------------------------------------------------------------------------------------------

plot_binclass_bg_points <- function(data)
{
	pal = c(lightbluegreen, "white", lightredorange)
	plot_bg(data, pal)
}


# ----------------------------------------------------------------------------------------------------------------------

plot_bg <- function(data, pal, own_breaks = FALSE)
{
	size = sqrt(nrow(data))

	x = sort(unique(data[ ,2]))
	y = sort(unique(data[ ,3]))

	if (own_breaks == FALSE)
		image(x, y, matrix(data[ ,1], nrow=size, ncol=size), col=pal) 
	else
	{
		number_of_colors = length(unique(pal))
		breaks = seq(0, number_of_colors) - 0.000001

		image(x, y, matrix(data[ ,1], nrow = size, ncol = size), col = pal, breaks = breaks) 
	}
}


# ----------------------------------------------------------------------------------------------------------------------



plot_1d_histogram_single <- function(name, size, grid_size)
{
	data1 = read.table(get_full_2d_filename(get_toy_name(name, size), "train"), sep=",")
	data2 = matrix(0 ,nrow=0, ncol=0)
	
	image_name = get_image_filename(name, "train", TRUE, paste("hrm", size, 1, sep="-"))
	jpeg(clean_name(image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)

	if (name == "dogs")
		pal = c(bluegreen, lightbluegreen)
	else
		pal = c(redorange, lightredorange)
	
	plot_1d_histogram(data1, data2, c(5, 2.5), pal)
	dev.off()
	
	
	density = read.table(get_full_1d_filename(get_toy_name(name, grid_size), "dens"), sep = ",")
	y_res = 10000
	aux_data = convert_density_to_aux_data(density, grid_size, y_res)
	
	intervals = c(5, 10, 20, 50, 100)
	for (i in 1:length(intervals))
	{
		image_name = get_image_filename(name, "dens", TRUE, paste("hrm", intervals[i], sep="-"))
		jpeg(clean_name(image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)
		plot_1d_histogram(aux_data, data2, c(1 / intervals[i], 1 / intervals[i]), pal, TRUE, 1)
		
		lines(density[ ,2], grid_size * density[ ,1], lwd=10)
		dev.off()
		
		
		if (name == "dogs")
			pal = c(bluegreen, lightbluegreen, darkbluegreen)
		else
			pal = c(redorange, lightredorange, darkredorange)
		
		image_name = get_image_filename(name, "train_dens", TRUE, paste("hrm", intervals[i], size, sep="-"))
		jpeg(clean_name(image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)
		plot_1d_histogram(aux_data, data1, c(1 / intervals[i], 1 / intervals[i]), pal, TRUE, 1)
		
		lines(density[ ,2], grid_size * density[ ,1], lwd=10)

		dev.off()
	}
}


# ----------------------------------------------------------------------------------------------------------------------



plot_1d_histogram_mixed <- function(size1, size2, steps)
{
	data1 = read.table(get_full_2d_filename(get_toy_name("dogs", size1), "train"), sep=",")
	data2 = read.table(get_full_2d_filename(get_toy_name("cats", size2), "train"), sep=",")

	image_name = get_image_filename("dogs-vs-cats", "train", TRUE, paste("hrm", 10/steps, sep=""))
	print(image_name)
	
	jpeg(clean_name(image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)

	pal = c(bluegreen, redorange, darkmess)

	
	decision_for_class_1 = plot_1d_histogram(data1, data2, c(steps, steps), pal)
	
	dev.off()
	
# 	-------


	image_name = get_image_filename("dogs-vs-cats", "train_dens", TRUE, paste("hrm", 10/steps, sep=""))
	print(image_name)
	
	density1 = read.table(get_full_1d_filename(get_toy_name("dogs", 500), "dens"), sep = ",")
	density2 = read.table(get_full_1d_filename(get_toy_name("cats", 500), "dens"), sep = ",")
	
	jpeg(clean_name(image_name), width = 2000, height = 2000, units = "px", pointsize = 30, quality = 100)

	decision_for_class_1 = plot_1d_histogram(data1, data2, c(steps / 50,  steps / 50), pal, TRUE, 1, c(1/3, 2/3))
	lines(density1[ ,2], 1/3 * 500 * density1[ ,1], lwd=10)
	lines(density2[ ,2], 2/3 * 500 * density2[ ,1], lwd=10)
	
	dev.off()
	
	
	data1 = read.table(get_full_2d_filename(get_toy_name("dogs", size1), "test"), sep=",")
	data2 = read.table(get_full_2d_filename(get_toy_name("cats", size2), "test"), sep=",")
	
	p1 <- hist(50 * data1[ ,1], breaks=seq(0, 50, steps))
	p2 <- hist(50 * data2[ ,1], breaks=seq(0, 50, steps))
	
	counts1 = p1$counts
	counts2 = p2$counts
	error = (sum(counts1[!decision_for_class_1]) + sum(counts2[decision_for_class_1])) / (nrow(data1) + nrow(data2))
	
	print(error)
}


# ----------------------------------------------------------------------------------------------------------------------


plot_1d_histogram <- function(data1, data2, steps, pal, normalize = FALSE, x_stretch_factor=50, weight_factors = c(1, 1))
{
	opar <- par(lwd=10)
	
	max_y = 240 * steps[1]
	xlab = "Gewicht in kg"
	

	p1 <- hist(x_stretch_factor * data1[ ,1], breaks=seq(0, x_stretch_factor, steps[1]))
	if (nrow(data2) > 0)
		p2 <- hist(x_stretch_factor * data2[ ,1], breaks=seq(0, x_stretch_factor, steps[2]))
		
		if (normalize == TRUE)
		{
			max_y = 6 * max(weight_factors)
			xlab = "Gewicht in 50kg"
			p1$counts = 1 / steps[1] * p1$counts / nrow(data1)
			if (nrow(data2) > 0)
				p2$counts = 1 / steps[1] * p2$counts / nrow(data2)
		}
		
	if (nrow(data2) > 0)
	{
		counts1 = p1$counts
		counts2 = p2$counts
		decision_for_class_1 = counts1 > counts2
	}
	else 
		decision_for_class_1 = 0
		
	p1$counts = weight_factors[1] * p1$counts
	if (nrow(data2) > 0)
		p2$counts = weight_factors[2] * p2$counts

	
	plot(p1, col = pal[1], main="", xlab=xlab, ylab="", lwd=10, cex.lab=2.0, cex.axis=2.0, cex.main=2.0, cex.sub=2.0, ylim=c(0, max_y))
	if (nrow(data2) > 0)
	{
		plot(p2, col = pal[2], add=T)
		if (length(pal) == 3)
		{
			p1$counts = pmin(p1$counts, p2$counts)
			plot(p1, col = pal[3], add=T)
		}
	}
	
	par(opar)
	
	return(decision_for_class_1)
}


# ----------------------------------------------------------------------------------------------------------------------


# value of the colors according to http://www.color-hex.com/

redorange = rgb(red = 255, green = 69, blue = 0, alpha = 255, maxColorValue = 255) 
bluegreen = rgb(red = 0, green = 186, blue = 255, alpha = 255, maxColorValue = 255) 

redorange_transparent = rgb(red = 255, green = 48, blue = 0, alpha = 200, maxColorValue = 255) 
bluegreen_transparent = rgb(red = 0, green = 186, blue = 255, alpha = 200, maxColorValue = 255) 

lightredorange = rgb(red = 255, green = 199, blue = 178, alpha = 255, maxColorValue = 255) 
lightbluegreen = rgb(red = 178, green = 234, blue = 255, alpha = 255, maxColorValue = 255) 

darkredorange = rgb(red = 127, green = 34, blue = 0, alpha = 255, maxColorValue = 255) 
darkbluegreen = rgb(red = 0, green = 93, blue = 127, alpha = 255, maxColorValue = 255) 
darkmess = rgb(red = 76, green = 76, blue = 76, alpha = 255, maxColorValue = 255) 
