// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.




#include <typeinfo>
using namespace std;

#include "sources/shared/basic_functions/flush_print.h"
#include "sources/shared/system_support/os_specifics.h"

#ifdef _WIN32
#ifndef __MINGW32__
	#pragma warning(disable:4996)
#endif
	#define _CRT_SECURE_NO_DEPRECATE
	#define _CRT_SECURE_NO_WARNINGS
#endif


//**********************************************************************************************************************************

template <typename Template_type> void file_read_entry(FILE* fp,  Template_type& input, string entry_name)
{
	file_read_entry_name(fp, entry_name);
	file_read(fp, input);
}


//**********************************************************************************************************************************


template <typename Template_type> void file_read_entry_vector(FILE* fp, vector <Template_type>& input, string entry_name)
{
	unsigned i;
	unsigned size;
	string complete_entry_name;
	
	if (entry_name.size() == 0)
		file_read(fp, input);
	else
	{
		complete_entry_name = entry_name + " length";
		file_read_entry(fp, size, complete_entry_name);
		input.resize(size);
		
		complete_entry_name = entry_name + " entries";
		file_read_entry_name(fp, complete_entry_name);
		for (i=0; i<size; i++)
			file_read(fp, input[i]);
	}
}



//**********************************************************************************************************************************


template <typename Template_type> void file_read(FILE* fp, vector <Template_type>& input)
{
	unsigned i;
	unsigned size;
	
	file_read(fp, size);
	input.resize(size);
	
	for (i=0; i<size; i++)
		file_read(fp, input[i]);
}

//**********************************************************************************************************************************


template <typename Template_type> void file_write_entry(FILE* fp,  Template_type& output, string entry_name)
{
	file_write_entry_name(fp, entry_name);
	file_write(fp, output);
}


//**********************************************************************************************************************************


template <typename Template_type> void file_write_entry_vector(FILE* fp, const vector <Template_type>& output, string entry_name)
{
	unsigned i;
	unsigned size;
	string complete_entry_name;
	const unsigned wrap_size = 100;
	
	
	if (entry_name.size() == 0)
		file_write(fp, output);
	else
	{
		complete_entry_name = entry_name + " length";
		size = unsigned(output.size());
		file_write_entry(fp, size, complete_entry_name);
		
		complete_entry_name = entry_name + " entries";
		file_write_entry_name(fp, complete_entry_name);
		if (output.size() > wrap_size)
			file_write_eol(fp);
		for (i=0; i<output.size(); i++)
		{
			file_write(fp, output[i]);
			if (((i + 1) % wrap_size == 0) and (output.size() > wrap_size))
				file_write_eol(fp);
		}
		
		if (output.size() > wrap_size)
			file_write_eol(fp);
	}
}

//**********************************************************************************************************************************


template <typename Template_type> void file_write(FILE* fp, vector <Template_type> output, string separator)
{
	unsigned i;
	
	if (fp != NULL)
	{
		file_write(fp, unsigned(output.size())); 
		file_write_eol(fp);
		
		for (i=0; i<output.size(); i++)
			file_write(fp, output[i], separator);
		if (output.size() > 0)
			file_write_eol(fp);
	}
}

