// Copyright 2017 Ingo Steinwart
//
// This file is part of liquidCluster.
//
// liquidCluster is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as 
// published by the Free Software Foundation, either version 3 of the 
// License, or (at your option) any later version.
//
// liquidCluster is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Affero General Public License for more details.

// You should have received a copy of the GNU Affero General Public License
// along with liquidCluster. If not, see <http://www.gnu.org/licenses/>.


#if !defined (WORKING_SET_CONTROL_CPP)
	#define WORKING_SET_CONTROL_CPP


#include "sources/shared/training_validation/working_set_control.h"


#include "sources/shared/basic_functions/basic_file_functions.h"


const string aux_identifier_ws_con = "working set control";



//**********************************************************************************************************************************


Tworking_set_control::Tworking_set_control()
{
	classification = true;

	size_of_tasks = 0;
	number_of_tasks = 0;
	
	working_set_selection_method = FULL_SET;
	partition_method = FULL_SET;
	number_of_covers = 1;
	reduce_covers = true;
	max_ignore_factor = 0.5;
	
	tree_reduction_factor = 2.0;
	max_tree_depth = 4;
	max_theoretical_node_width = 20;
	
	size_of_cells = 0;
	number_of_cells = 1;
	size_of_dataset_to_find_partition = 0;
	radius = 0.0;
	random_seed = -1;
};

//**********************************************************************************************************************************


void Tworking_set_control::set_partition_method_with_defaults(unsigned partition_method)
{
	Tworking_set_control::partition_method = partition_method;
	switch (partition_method)
	{
		case NO_PARTITION:
			break;
				
		case RANDOM_CHUNK_BY_SIZE:
			size_of_cells = 2000;
			break;
					
		case RANDOM_CHUNK_BY_NUMBER:
			number_of_cells = 10;
			break;
			
		case VORONOI_BY_RADIUS:
			radius = 1.0;
			size_of_dataset_to_find_partition = 0;
			size_of_dataset_to_find_partition = 0;
			break;
			
		case VORONOI_BY_SIZE:
			size_of_cells = 2000;
			reduce_covers = true;
			size_of_dataset_to_find_partition = 50000;
			break;
			
		case OVERLAP_BY_SIZE:
			size_of_cells = 2000;
			max_ignore_factor = 0.5;
			size_of_dataset_to_find_partition = 50000;
			number_of_covers = 1;
			break;
			
		case VORONOI_TREE_BY_SIZE:
			size_of_cells = 2000;
			reduce_covers = true;
			size_of_dataset_to_find_partition = 50000;
			number_of_covers = 1;
			tree_reduction_factor = 2.0;
			max_tree_depth = 4;
			max_theoretical_node_width = 20;
			break;
			
		default:
			flush_exit(ERROR_DATA_MISMATCH, "Trying to use partition method %d% that does not exist.", partition_method);
	}
}


//**********************************************************************************************************************************

void Tworking_set_control::write_to_file(FILE *fp) const
{
	file_write_aux_identifier(fp, aux_identifier_ws_con);
	
	file_write_entry(fp, classification, "classification");

	file_write_entry(fp, size_of_tasks, "size of tasks");
	file_write_entry(fp, number_of_tasks, "number of tasks");
	file_write_entry(fp, working_set_selection_method, "working set method");
	file_write_entry(fp, partition_method, "partition method");
	file_write_entry(fp, number_of_covers, "number of covers");
	file_write_entry(fp, reduce_covers, "reduce covers");
	file_write_entry(fp, max_ignore_factor, "max ignore factor");
	
	file_write_entry(fp, size_of_cells, "size of cells");
	file_write_entry(fp, number_of_cells, "number of cells");
	file_write_entry(fp, size_of_dataset_to_find_partition, "subset size");
	file_write_entry(fp, radius, "radius");
	file_write_entry(fp, random_seed, "random seed");
	
	file_write_eol(fp);
};


//**********************************************************************************************************************************


void Tworking_set_control::read_from_file(FILE *fp)
{
	file_read_aux_identifier(fp, aux_identifier_ws_con);
	
	file_read_entry(fp, classification, "classification");

	file_read_entry(fp, size_of_tasks, "size of tasks");
	file_read_entry(fp, number_of_tasks, "number of tasks");
	file_read_entry(fp, working_set_selection_method, "working set method");
	file_read_entry(fp, partition_method, "partition method");
	file_read_entry(fp, number_of_covers, "number of covers");
	file_read_entry(fp, reduce_covers, "reduce covers");
	file_read_entry(fp, max_ignore_factor, "max ignore factor");
	
	file_read_entry(fp, size_of_cells, "size of cells");
	file_read_entry(fp, number_of_cells, "number of cells");
	file_read_entry(fp, size_of_dataset_to_find_partition, "subset size");
	file_read_entry(fp, radius, "radius");
	file_read_entry(fp, random_seed, "random seed");
}

#endif

